/***************************************************************************
                          kpgserver.cpp  -  description
                             -------------------
    begin                : � led 6 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgserver.h"

// include files for Qt
#include <qtable.h>
#include <qstringlist.h>
#include <qregexp.h>

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgdatabase.h"
#include "kpgconnection.h"
#include "kpgusersfolder.h"
#include "kpggroupsfolder.h"
#include "kpgtablespacesfolder.h"
#include "../PropertyPages/kpgserverproppage.h"
#include "../kpgsqldialog.h"

KPGServer::KPGServer(QListView *parent)
  : KPGTreeItem(parent)
{
  setPixmap(0, *m_pIconServer);

  m_pDbConnection = 0;
	
	m_pFolderUsers = 0;
	m_pFolderGroups = 0;
	m_pFolderTableSpaces = 0;
  
  m_uiVersionMinor = 0;
  m_uiVersionMiddle = 0;
  m_uiVersionMajor = 0;
}

KPGServer::~KPGServer()
{
  if(m_pDbConnection)
    {
		  if(m_pDbConnection->is_open() == true)
				m_pDbConnection->disconnect();
 
		  delete m_pDbConnection;
		}
}

bool KPGServer::connectToServer(
  const QString strHost,
  const QString strPort,
  const QString strDatabase,
  const QString strUser,
  const QString strPassword
 )
{
  QString strConnectString = (strDatabase.length() == 0) ?
    makeConnectionString(strHost, strPort, strUser, strPassword)
    :
    makeConnectionString(strHost, strPort, strDatabase, strUser, strPassword)
    ;
  
  try
  {
    m_pDbConnection = new KPGConnection(strConnectString);
    m_strPassword = strPassword;
  }
  catch (const std::exception &e)
  {
    kdError() << "Failed to open database " << e.what() << endl;
    KMessageBox::sorry(0, e.what());
    return false;
  }

  if(m_pDbConnection->is_open() == false)
  {
    kdError() << "Failed to open database" << endl;
    KMessageBox::sorry(0, "Failed to open connection to server " + strHost);
    return false;
  }
  
  setText(0, strHost);
  
  getVersionInfo(m_pDbConnection);

  refresh();
	  
  return true;
}

bool KPGServer::refresh()
{
	// delete all child items
  while(QListViewItem * pItem = firstChild())
    delete pItem; 
	
	// obtain list of tables
  pqxx::work pqxxXaction(*m_pDbConnection, "refresh server");
		
	refreshDatabaseList(pqxxXaction, false);
	
	m_pFolderUsers = new KPGUsersFolder(this);
	m_pFolderGroups = new KPGGroupsFolder(this);
		
	m_pFolderUsers->refresh(pqxxXaction);
	m_pFolderGroups->refresh(pqxxXaction);
	
	if(m_uiVersionMajor >= 8)
	{
	  m_pFolderTableSpaces = new KPGTablespacesFolder(this); // Tablespaces suppor introduced in PostgreSQL 8
		m_pFolderTableSpaces->refresh(pqxxXaction);
	}
		
  setOpen(true);
	return true;
}

// Intelligent refresh - find new or removed databases
bool KPGServer::smartRefresh()
{
	// obtain list of tables
  pqxx::work pqxxXaction(*m_pDbConnection, "refresh server");
	refreshDatabaseList(pqxxXaction, true);
	return true;
}

bool KPGServer::getVersionInfo(KPGConnection *pDbConnection)
{
  // obtain version info
  pqxx::work pqxxXaction(*pDbConnection, "getVersionInfo");

	QString strQuery("SELECT version();");
	
	pqxx::result pqxxResult; 
  try
  {
    pqxxResult = pqxxXaction.Exec(strQuery.latin1());
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 

  if (pqxxResult.size() >= 1)
  {
    m_strVersionInfo = pqxxResult[0][0].c_str();
  }
  
  // Parse version
  
  //--- major
  QRegExp patternMajor("PostgreSQL \\d+", false); // major version
               
  int pos=patternMajor.search(m_strVersionInfo, 0);
  if(pos >= 0)
  {
    int l = patternMajor.matchedLength();
    QString s(m_strVersionInfo.mid(pos + 11, l - 11));
    
    m_uiVersionMajor = s.toInt();
    
    //--- middle
    QRegExp patternNumber("\\d+", false); // middle version
                
    pos=patternNumber.search(m_strVersionInfo, pos + l);
    if(pos >= 0)
    {
      l = patternNumber.matchedLength();
      s = m_strVersionInfo.mid(pos, l);
      
      m_uiVersionMiddle = s.toInt();
      
      // minor
      pos=patternNumber.search(m_strVersionInfo, pos + l);
      if(pos >= 0)
      {
        l = patternNumber.matchedLength();
        s = m_strVersionInfo.mid(pos, l);
      
        m_uiVersionMinor = s.toInt();
      }
    }
  } 
  else
    {
      KMessageBox::sorry(0, "Cannot parse Version info");
      kdError() << "Cannot parse Version info" << endl;
      return false;
    }
       
  return true;
}

bool KPGServer::refreshDatabaseList(pqxx::work &pqxxXaction, bool bSmart)
{
  // obtain list of databases
  
  QString strQuery("SELECT db.oid, datname, pg_get_userbyid(datdba) AS datowner, datacl");
  
  if(m_uiVersionMajor <= 7)
    strQuery.append(", datpath "); // deprecated in PostgreSQL 8
  else
     strQuery.append(", dattablespace, spcname ");
     
  strQuery.append(", pg_encoding_to_char(encoding) AS serverencoding, ");
  strQuery.append("has_database_privilege(db.oid, 'CREATE') as cancreate, datconfig, datallowconn, datistemplate ");
      
  strQuery.append("FROM pg_database db ");
  
  if(m_uiVersionMajor >= 8)
    strQuery.append("LEFT JOIN pg_tablespace ts ON db.dattablespace=ts.oid ");

  strQuery.append("ORDER BY datname");  
    
	try
  {
    m_pqxxResultDatabases = pqxxXaction.Exec(strQuery.latin1());
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 

	// Remove deleted databases from Server childrens list
	QStringList strListOfExistingDatabases;
	for (result::size_type i = 0; i != m_pqxxResultDatabases.size(); ++i)
  	strListOfExistingDatabases.append(QString(m_pqxxResultDatabases[i]["datname"].c_str()));
	
	QListViewItem * pItem = firstChild();
	while(pItem)
	{
		if((static_cast <KPGTreeItem *> (pItem))->type() == KPGTreeItem::nodeDatabase)
		{
			if(strListOfExistingDatabases.contains(pItem->text(0)) == false)
			{
				delete pItem; // database not exists - delete it
				pItem = firstChild(); // restart cycle
				continue;
			}
		}			
		pItem = pItem->nextSibling();
	}
	
  KPGDatabase *pDatabase = 0;
  
	// Append databases to Server childrens list
  for (result::size_type i = 0; i != m_pqxxResultDatabases.size(); ++i)
  {
		pqxx::oid oid;
		m_pqxxResultDatabases[i][0].to(oid);
		
		bool bCreateNew = true; // default
		if(bSmart)
		{
			// look, if this database already exists
			pItem = firstChild();
			while(pItem)
			{
				if((static_cast <KPGTreeItem *> (pItem))->type() == KPGTreeItem::nodeDatabase)
				{
					if(pItem->text(0) == QString(m_pqxxResultDatabases[i]["datname"].c_str()))
					{
						bCreateNew = false; // database exists, skip creation
						break;
					}
				}			
				pItem = pItem->nextSibling();
			}
		}
		
		if(bCreateNew)
		{
      if(pDatabase == 0)
			 pDatabase = new KPGDatabase(
					this,
					m_pqxxResultDatabases[i]["datname"].c_str(),
					oid
					);
			else
        pDatabase = new KPGDatabase(
          this,
          pDatabase,
          m_pqxxResultDatabases[i]["datname"].c_str(),
          oid
          );
      
			bool bCanCreate;
			bool bAllowConnections;
			bool bIsTemplate;
					
			m_pqxxResultDatabases[i]["cancreate"].to(bCanCreate);
			m_pqxxResultDatabases[i]["datallowconn"].to(bAllowConnections);
			m_pqxxResultDatabases[i]["datistemplate"].to(bIsTemplate);
			
      if(m_uiVersionMajor <= 7)
      {							
			   pDatabase->setProperties(
				    m_pqxxResultDatabases[i]["datowner"].c_str(), 
				    m_pqxxResultDatabases[i]["datacl"].c_str(), 
				    m_pqxxResultDatabases[i]["datpath"].c_str(), 
				    m_pqxxResultDatabases[i]["serverencoding"].c_str(), 
				    bCanCreate,
				    m_pqxxResultDatabases[i]["datconfig"].c_str(), 
				    bAllowConnections,
				    bIsTemplate
				    );
      }
      else
      {  
         pqxx::oid oidTablespace;
         m_pqxxResultDatabases[i]["dattablespace"].to(oidTablespace);
                  
         pDatabase->setProperties(
            m_pqxxResultDatabases[i]["datowner"].c_str(), 
            m_pqxxResultDatabases[i]["datacl"].c_str(),
            oidTablespace, 
            m_pqxxResultDatabases[i]["spcname"].c_str(), 
            m_pqxxResultDatabases[i]["serverencoding"].c_str(), 
            bCanCreate,
            m_pqxxResultDatabases[i]["datconfig"].c_str(), 
            bAllowConnections,
            bIsTemplate
            );
      }
		}
  }
  return true;
}

result KPGServer::runQuery(const QString strQuery)
{
  // run query
  work pqxxXaction(*m_pDbConnection, "runQuery");
  return pqxxXaction.exec(strQuery.latin1());
}

const QString KPGServer::url() const 
{ 
  if(m_pDbConnection == 0) return "";
  
  return QString("host=" + QString(m_pDbConnection->hostname()) +
	               " port=" + QString(m_pDbConnection->port()) +
                 " dbname=" + QString(m_pDbConnection->dbname()) +
	               " user=" + QString(m_pDbConnection->username())
                ); 
}

void KPGServer::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGServerPropPage *pPropertyPage = static_cast <KPGServerPropPage *> (widgetProperies);
	
	pPropertyPage->setVersionInfo(m_strVersionInfo);
	
	//----------------------------------------------------------------------------
	//Display databases list
		
  QTable *pTableDatabases = pPropertyPage->m_pTableDatabases;
  
  // fill properties table with result data
  int nTableCols = fillPropertiesTable(m_pqxxResultDatabases, pTableDatabases);
  
  // field names
  QHeader* hHeader = pTableDatabases->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Owner"));
  hHeader->setLabel(nCol++, i18n("ACL"));
  
  if(m_uiVersionMajor <= 7)
    hHeader->setLabel(nCol++, i18n("Path"));
  else
    hHeader->setLabel(nCol++, i18n("Tablespace"));
    
  hHeader->setLabel(nCol++, i18n("Encoding"));
  hHeader->setLabel(nCol++, i18n("Can create"));
  hHeader->setLabel(nCol++, i18n("Default variables"));
  hHeader->setLabel(nCol++, i18n("Allow connection"));
  hHeader->setLabel(nCol++, i18n("Is template"));
	
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableDatabases->adjustColumn(nCol);
  }
}

